import logging
import sys
from logging import Formatter
from logging import LogRecord
from logging import StreamHandler


class ColorFormatter(logging.Formatter):
    grey = "\x1b[38;20m"
    blue = "\x1b[38;5;39m"
    yellow = "\x1b[33;20m"
    red = "\x1b[31;20m"
    bold_red = "\x1b[31;1m"
    reset = "\x1b[0m"

    colors = {
        logging.DEBUG: grey,
        logging.INFO: blue,
        logging.WARNING: yellow,
        logging.ERROR: red,
        logging.CRITICAL: bold_red,
    }

    def format(self, record: LogRecord):
        color = self.colors.get(record.levelno, self.grey)
        message = super().format(record)
        return color + message + self.reset


# intialize logger
root = logging.getLogger("pkg_updater")
root.setLevel(logging.DEBUG)

# formatter
fmt = "%(asctime)s - %(levelname)s - %(message)s"
datefmt = "%m/%d/%Y %I:%M:%S %p"
formatter = Formatter(fmt, datefmt)
color_formatter = ColorFormatter(fmt, datefmt)

# stream handler
stream_handler = StreamHandler(sys.stdout)
stream_handler.setFormatter(color_formatter)
stream_handler.setLevel(logging.INFO)
root.addHandler(stream_handler)


# ---------------------------------------------------------------------------
# Utility functions at module level.
# Basically delegate everything to the root logger.
# # ---------------------------------------------------------------------------
def critical(msg: str):
    root.critical(msg)


def error(msg: str):
    root.error(msg)


def exception(msg: str):
    root.exception(msg)


def warning(msg: str):
    root.warning(msg)


def info(msg: str):
    root.info(msg)


def debug(msg: str):
    root.debug(msg)



from argparse import Namespace
from subprocess import DEVNULL
from subprocess import PIPE
from subprocess import Popen
from subprocess import check_output
import subprocess
from threading import Condition

import psutil
from PyQt6.QtCore import QThread

from pkg_updater import logger

# hide flashing console when using pythonw
startupinfo = subprocess.STARTUPINFO()
startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
startupinfo.wShowWindow = subprocess.SW_HIDE


def get_running_processes(name: str, cmdline: str):
    proc_iter = psutil.process_iter()
    processes: list[psutil.Process] = []
    for i in proc_iter:
        try:
            i_cmdline = " ".join(i.cmdline())
            if "--restart" in i_cmdline:
                # filter self
                continue
            if name in i.name() and cmdline in i_cmdline:
                processes.append(i)
        except psutil.Error:
            pass
    return processes


def get_is_up_to_date(pkg_name: str, extra_index_url: str = ""):
    cmd = ["pip", "install", "--upgrade", pkg_name]
    if extra_index_url:
        cmd += ["--extra-index-url", extra_index_url]
    cmd += ["--dry-run"]
    logger.info("Checking for updates...")
    stdout = check_output(cmd, stderr=PIPE, startupinfo=startupinfo).decode()
    return "Would install" not in stdout


def install_updates(pkg_name: str, extra_index_url: str = ""):
    cmd = ["pip", "install", "--upgrade", pkg_name]
    if extra_index_url:
        cmd += ["--extra-index-url", extra_index_url]

    logger.info("Installing updates...")
    with Popen(
        cmd, stdout=PIPE, stderr=PIPE, text=True, startupinfo=startupinfo
    ) as p:
        if p.stdout:
            for line in p.stdout:
                line = line.strip()
                if line:
                    logger.info(line)
        if p.stderr:
            for line in p.stderr:
                line = line.strip()
                if line and "[notice]" not in line:
                    logger.info(line)


class UpdaterThread(QThread):
    def __init__(self, args: Namespace, update_condition: Condition):
        super().__init__()
        self.update_condition = update_condition
        self.args = args

    def run(self):
        logger.info(f"Next update in {self.args.delay_first}s...")
        with self.update_condition:
            self.update_condition.wait(self.args.delay_first)
        while True:
            try:
                if get_is_up_to_date(
                    self.args.package_name, self.args.extra_index_url
                ):
                    logger.info("Already up to date.")
                else:
                    if self.args.restart and self.args.process_name:
                        logger.info(
                            "Gracefully shutting down running processes..."
                        )
                        processes = get_running_processes(
                            self.args.process_name, self.args.process_cmdline
                        )
                        closed_processes = [
                            (i.cmdline(), i.cwd()) for i in processes
                        ]
                        for i in processes:
                            logger.info(f"Closing process: {i.name()}")
                            i.terminate()
                    else:
                        closed_processes = []
                    install_updates(
                        self.args.package_name, self.args.extra_index_url
                    )
                    if self.args.restart and self.args.process_name:
                        logger.info("Restarting closed processes...")
                        for cmd, cwd in closed_processes:
                            Popen(
                                cmd,
                                cwd=cwd,
                                stderr=DEVNULL,
                                stdout=DEVNULL,
                                startupinfo=startupinfo,
                            )
                    logger.info("Complete.")
            except Exception:
                logger.exception("Unhandled exception occurred.")
            logger.info(f"Next update in {self.args.interval}s...")
            with self.update_condition:
                self.update_condition.wait(self.args.interval)



import ctypes
import logging
from pathlib import Path
import sys
from argparse import ArgumentParser
from argparse import Namespace
from logging import Formatter
from threading import Condition
from typing import Optional

from PyQt6.QtCore import QCoreApplication
from PyQt6.QtGui import QAction
from PyQt6.QtGui import QCloseEvent
from PyQt6.QtGui import QFont
from PyQt6.QtGui import QIcon
from PyQt6.QtWidgets import QApplication
from PyQt6.QtWidgets import QMainWindow
from PyQt6.QtWidgets import QMenu
from PyQt6.QtWidgets import QPlainTextEdit
from PyQt6.QtWidgets import QPushButton
from PyQt6.QtWidgets import QSystemTrayIcon
from PyQt6.QtWidgets import QVBoxLayout
from PyQt6.QtWidgets import QWidget

from pkg_updater.handlers import PyQtHandler
from pkg_updater.logger import root
from pkg_updater.updater import UpdaterThread

# fix taskbar icon for some windows version
ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(
    "sandbox.pkgupdater"
)


class MainWindow(QMainWindow):
    def __init__(self, instance: QCoreApplication, args: Namespace):
        super().__init__()

        # window configuration
        self.icon = QIcon(str(Path(__file__).parent / "favicon.svg"))
        self.setWindowTitle("Package Updater")
        self.setWindowIcon(self.icon)

        # tray actions
        minimize_action = QAction("Minimize", self)
        minimize_action.triggered.connect(self.hide)
        restore_action = QAction("Restore", self)
        restore_action.triggered.connect(self.showNormal)
        quit_action = QAction("Quit", self)
        quit_action.triggered.connect(instance.quit)

        # tray menu
        tray_menu = QMenu()
        tray_menu.addAction(minimize_action)
        tray_menu.addAction(restore_action)
        tray_menu.addSeparator()
        tray_menu.addAction(quit_action)

        # tray
        self.tray_icon = QSystemTrayIcon(self)
        self.tray_icon.setIcon(self.icon)
        self.tray_icon.setVisible(True)
        self.tray_icon.show()
        self.tray_icon.setToolTip("Package Updater")
        self.tray_icon.setContextMenu(tray_menu)
        self.tray_icon.activated.connect(self.on_tray_activated)

        # widgets
        self.update_cv = Condition()
        self.check_for_updates = QPushButton("Check for updates")
        self.check_for_updates.clicked.connect(self.on_update)
        self.console = QPlainTextEdit()
        self.console.setReadOnly(True)
        self.console.setFont(QFont("Lucida Console", 9))

        # layout
        layout = QVBoxLayout()
        layout.addWidget(self.check_for_updates)
        layout.addWidget(self.console)
        widget = QWidget()
        widget.setLayout(layout)
        self.setCentralWidget(widget)

        # logger configuration
        formatter = Formatter("%(asctime)s %(message)s", "%H:%M:%S")
        handler = PyQtHandler()
        handler.setFormatter(formatter)
        handler.setLevel(logging.INFO)
        handler.log.connect(self.console.appendHtml)
        root.addHandler(handler)

        # threads
        self.updater = UpdaterThread(args, self.update_cv)
        self.updater.start()

    def on_tray_activated(self, reason: QSystemTrayIcon.ActivationReason):
        if reason == QSystemTrayIcon.ActivationReason.DoubleClick:
            self.show()

    def on_update(self):
        with self.update_cv:
            self.update_cv.notify_all()

    def closeEvent(self, a0: Optional[QCloseEvent]):
        if a0:
            a0.ignore()
        self.hide()


def main():
    parser = ArgumentParser()
    parser.add_argument("package_name")
    parser.add_argument("--extra-index-url", default="")
    parser.add_argument("--interval", default=900, type=int)
    parser.add_argument("--delay-first", default=900, type=int)
    parser.add_argument("--restart", action="store_true")
    parser.add_argument("--process-name", default="")
    parser.add_argument("--process-cmdline", default="")
    args = parser.parse_args()
    app = QApplication([])
    window = MainWindow(app, args)
    window.show()
    sys.exit(app.exec())


if __name__ == "__main__":
    main()



import logging
from logging import Handler
from logging import LogRecord

from PyQt6.QtCore import QObject
from PyQt6.QtCore import pyqtSignal

COLORS = {
    logging.DEBUG: "#0071A6",
    logging.INFO: "#0071A6",
    logging.WARNING: " #6B6D00",
    logging.ERROR: "#D43131",
    logging.CRITICAL: "#D43131",
}


class PyQtHandler(Handler, QObject):
    log = pyqtSignal(str)

    def __init__(self):
        Handler.__init__(self)
        QObject.__init__(self)

    def emit(self, record: LogRecord):
        msg = self.format(record)
        color = COLORS[record.levelno]
        msg = msg.replace("\n", "<br/>")
        self.log.emit(f"<span style='color: {color};'>{msg}</span>")
